#!/usr/bin/env php
<?php
/**
 * @author ElisDN <mail@elisdn.ru>
 * @link http://www.elisdn.ru/blog/58/old-posts-promotion-on-twitter
 * @version 1.0
 */

namespace Parser;

use Cache\FileCache;
use Logger\ConsoleLogger;
use Shorter\BitLy;
use Shorter\CachedShorter;
use Shorter\DummyShorter;
use Twitter\Tagger;
use Twitter\Tweet;

require(dirname(__FILE__) . '/inc/autoload.php');

// Вывод информации на экран
$logger = new ConsoleLogger();

#############################

// Проверка на правильность вызова
if (empty($argv[1]) || empty($argv[2])) {
	$logger->writelnNotice('php tweets.php <config_file> <result_file> [--reset]');
	die(1);
}

// Получение переданных параметров
$config_file = $argv[1];
$result_file = $argv[2];
$reset_cache = in_array('--reset', $argv);

// Проверка и загрузка файла конфигурации
if (!file_exists($config_file)) {
	$logger->writelnError('Configuration file is not found');
	die(1);
}

$config = require($config_file);

#############################

// Для записи твитов в файл
$result = new Result($result_file);

// Генератор хэштегов
$tagger = new Tagger($config['tags']['required'], $config['tags']['matched']);

// Кеширование уже полученных ранее данных
$cache = new FileCache($config['cache_file']);

// Если передали флаг очистки кеша
if ($reset_cache) {
	$cache->flush();
}

// Включить окращатель URL если указан ключ
if (!empty($config['bitly_key'])) {
	$shorter = new CachedShorter(new BitLy($config['bitly_key']), $cache);
} else {
	$shorter = new DummyShorter();
}

#############################

$sitemap = new Sitemap();
// Загружаем карту сайта
if ($sitemap->load($config['sitemap'])) {
	// Обходим адреса
	foreach ($sitemap->getUrls() as $url) {
		// Проверяем по списку исключений
		if (preg_match($config['match'], $url) && !preg_match($config['exclude'], $url)) {
			// Создаём экземпляр страницы для текущего адреса
			$page = new CachedHTMLPage(new HTMLPage(), $cache);
			// Если страница загружена
			if ($page->load($url)) {
				// Сокращаем адрес
				$short_url = $shorter->shortUrl($url);
				// Извлекаем из страницы элементы
				$h1 = $page->getH1();
				$title = $page->getTitle();
				$description = $page->getDescription();
				$keywords = $page->getKeywords();
				// Получаем теги на основе ключевых слов
				$words = $h1 . ' ' . $title . ' ' . $description . ' ' . $keywords;
				$tags = $tagger->getTags($words);
				// Передаём данные в конструктор твита
				$tweet = new Tweet($h1, $short_url, $tags, $config['via'], $config['prefix']);
				// Генерируем сообщение
				$text = $tweet->getMessage();
				// Записываем готовый твит в файл и выводим на экран
				$result->add($text);
				$logger->write($text);
				// Показываем корректность длины твита
				$length = mb_strlen($text, 'UTF-8');
				if ($length <= 140) {
					$logger->writelnSuccess($length);
				} else {
					$logger->writelnError($length);
				}
			} else {
				$logger->writelnError($url);
			}
		}
	}
} else {
	$logger->writelnError('Sitemap is not loaded');
	die(1);
}